#! /usr/bin/perl -Idebian/tests/lib

use diagnostics;
use strict;
use warnings;

use AdduserTestsCommon;
use Test::More;

assert_path_exists("/");

done_testing();

# vim: tabstop=4 shiftwidth=4 expandtab
